# Copyright (C) 2006-2010 Red Hat, Inc.
#
# This program is free software; you can redistribute
# it and/or modify it under the terms of version 2 of the
# GNU General Public License as published by the
# Free Software Foundation.

from TagObject import TagObject

TAG_NAME = 'totem'

class Totem(TagObject):
  DEFAULTS = {
    'join':                                 '50',
    'token':                                '1000',
    'token_retransmits_before_loss_const':  '4',
    'consensus':                            '1200',
  }

  def __init__(self):
    TagObject.__init__(self)
    self.TAG_NAME = TAG_NAME

  def getJoinTimeout(self):
    return self.getAttribute('join')

  def setJoinTimeout(self, val):
    return self.addIntegerAttribute('join', val)

  def delJoinTimeout(self):
    return self.removeAttribute('join')

  def getTokenTimeout(self):
    return self.getAttribute('token')

  def setTokenTimeout(self, val):
    return self.addIntegerAttribute('token', val)

  def delTokenTimeout(self):
    return self.removeAttribute('token')

  def getTokenRetransmits(self):
    return self.getAttribute('token_retransmits_before_loss_const')

  def setTokenRetransmits(self, val):
    return self.addIntegerAttribute('token_retransmits_before_loss_const', val)

  def delTokenRetransmits(self):
    return self.removeAttribute('token_retransmits_before_loss_const')

  def getConsensusTimeout(self):
    return self.getAttribute('consensus')

  def setConsensusTimeout(self, val):
    return self.addIntegerAttribute('consensus', val)

  def delConsensusTimeout(self):
    return self.removeAttribute('consensus')

  def getRRPMode(self):
    return self.getAttribute('rrp_mode')

  def setRRPMode(self, val):
    if not val:
        self.removeAttribute('rrp_mode')
        return None

    val = val.lower()
    if val not in ('active', 'passive', 'none'):
        raise ValueError, val
    self.addAttribute('rrp_mode', val)
    return val

  def delRRPMode(self):
    return self.removeAttribute('rrp_mode')

  def getSecAuth(self):
    return self.getBinaryAttribute('secauth')

  def setSecAuth(self, val):
    return self.addBinaryAttribute('secauth', val, ('off', 'on'))

  def delSecAuth(self):
    return self.removeAttribute('secauth')

  def getFailRecvConst(self):
    return self.getAttribute('fail_recv_const')

  def setFailRecvConst(self, val):
    return self.addIntegerAttribute('fail_recv_const', val)

  def delFailRecvConst(self):
    return self.removeAttribute('fail_recv_const')

  def getKeyFile(self):
    return self.getAttribute('keyfile')

  def setKeyFile(self, val):
    return self.addAttribute('keyfile', val)

  def delKeyFile(self):
    return self.removeAttribute('keyfile')
