from zope.interface import implements

from repoze.who.interfaces import IAuthenticator, IMetadataProvider
from repoze.who.utils import resolveDotted
import lucipam

class PAMAuthenticatorPlugin(object):
    implements(IAuthenticator,IMetadataProvider)

    def __init__(self,
                 serverfqdn=None,
                 user_realm=None,
                 iplocalport=None,
                 ipremoteport=None):
        self.serverfqdn = serverfqdn
        self.user_realm = user_realm
        self.iplocalport = iplocalport
        self.ipremoteport = ipremoteport

    # IAuthenticatorPlugin
    def authenticate(self, environ, identity):
        if not ('login' in identity and 'password' in identity):
            return None

        if lucipam.checkauth(identity['login'], identity['password'], self.serverfqdn, self.user_realm, self.iplocalport, self.ipremoteport):
            return identity['login']

        return None

    def add_metadata(self, environ, identity):
        username = identity.get('repoze.who.userid')
        groups = ['managers']
        permissions = ['manage']
        identity['user'] = username
        identity['groups'] = groups
        identity['permissions'] = permissions
        if 'repoze.what.credentials' not in environ:
            environ['repoze.what.credentials'] = {}
        environ['repoze.what.credentials']['groups'] = groups
        environ['repoze.what.credentials']['permissions'] = permissions
        # Adding the userid:
        userid = identity['repoze.who.userid']
        environ['repoze.what.credentials']['repoze.what.userid'] = userid

def make_pam_authenticator(serverfqdn=None,
                           user_realm=None,
                           iplocalport=None,
                           ipremoteport=None):
    """
    Configure :class:`PAMAuthenticatorPlugin`.
    """
    return PAMAuthenticatorPlugin(serverfqdn, user_realm, iplocalport, ipremoteport)

from repoze.what.plugins.pylonshq import booleanize_predicates
booleanize_predicates()
